/***************************************************************************
                          kiwriteview.cpp  -  description                              
                             -------------------                                         
    begin                : Fri Oct  8 19:11:34 PDT 1999
                                           
    copyright            : (C) 1999 by Subhabrata Biswas                         
    email                : subhabrata.biswas@usa.net                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include <kiwriteview.h>
#include <common.h>
#include "transliterator.h"

QFont    gEngFont ;
QFont    gIndFont ;

KIWriteView::KIWriteView(KApplication* a,KIWriteDoc* doc,QWidget *parent)
	: QWidget(parent),
		m_tab(this),
		m_ind(kapp, this, "english"),
		m_eng(kapp, this, "indian")
{
	m_eng.setFont(gEngFont) ;
	m_ind.setFont(gIndFont) ;
	
	m_tab.addTab(&m_eng, "En&glish") ;
	m_tab.addTab(&m_ind, "&Indian") ;
	
	m_currTab = 0 ;
	connect(&m_tab, SIGNAL(tabSelected(int)), this, SLOT(tabSelected(int))) ;
	
	m_ind.setReadOnly(true) ;
	m_eng.setFocus() ;
}

KIWriteView::~KIWriteView()
{
}


void KIWriteView::resizeEvent(QResizeEvent *rsz)
{
	m_tab.resize(rsz->size()) ;
}

void KIWriteView::saveFile()
{
	m_doc->saveContents(&m_eng) ;
}

void KIWriteView::loadFile()
{
	m_doc->loadContents(&m_eng) ;
}

void KIWriteView::saveAs()
{
	m_doc->saveAs(&m_eng) ;
}

void KIWriteView::closeFile()
{
	m_doc->closeFile(&m_eng) ;
}

void KIWriteView::newFile()
{
	m_doc->closeFile(&m_eng) ;
}

KEdit & KIWriteView::CurrentView()
{
	return (m_currTab == 0) ? m_eng : m_ind ;
}

void KIWriteView::tabSelected(int i)
{
	CTransliterator    trans(true) ;
	char               in[1024], out[1024] ;
	int                j ;
	
	m_currTab = i ;
	
	if (i == 1)
	{
		m_ind.newFile() ;
		
		for (j = 0 ; j < m_eng.numLines() ; j++)
		{
			strcpy(in, m_eng.textLine(j)) ;
			memset(out, 0, sizeof(out)) ;
			trans.ForwardConvert(in, out) ;
			m_ind.insertLine(out) ;
		}
		m_ind.setFocus() ;
	}
	else
		m_eng.setFocus() ;
}

void KIWriteView::fontChanged()
{
	kapp->getConfig()->setGroup("FontSpecs") ;
	if (m_currTab == 0)
	{
		gEngFont = m_eng.font() ;
		kapp->getConfig()->writeEntry("EnglishFont", gEngFont) ;
	}
	else
	{
		gIndFont = m_ind.font() ;
		kapp->getConfig()->writeEntry("IndianFont", gIndFont) ;
	}	
	kapp->getConfig()->setGroup("") ;
}

void KIWriteView::print(QPainter &paint)
{
	KEdit   *edit = (m_currTab == 0) ? &m_eng : &m_ind ;
	int      i ;
	char    *line ;
	QRect    view ;
	int      len ;
	int      width, height ;
	QRect    act ;
	int      tf = AlignLeft | AlignTop | ExpandTabs | WordBreak ;
	
	paint.setFont(edit->font()) ;
	
	view = paint.viewport() ;
	width = view.width() ;
	height = view.height() ;
	for (i = 0 ; i < edit->numLines() ; i++)
	{
		line = (char *)edit->textLine(i) ;
		len = strlen(line) ;
		
		paint.drawText(view.left(), view.height() - height, view.right(), height, tf, line, len, &act, NULL) ;
		height -= act.height() ;
	}
}

void KIWriteView::editCut()
{
	KEdit   *edit = (m_currTab == 0) ? &m_eng : &m_ind ;

	if (m_currTab == 0)
		edit->cut() ;
	else
		edit->copyText() ;
}

void KIWriteView::editCopy()
{
	KEdit   *edit = (m_currTab == 0) ? &m_eng : &m_ind ;
	
	edit->copyText() ;
}

void KIWriteView::editPaste()
{
	if (m_currTab != 0)
		return ;
		
	m_eng.paste() ;
}

void KIWriteView::editSelectAll()
{
	KEdit   *edit = (m_currTab == 0) ? &m_eng : &m_ind ;
	
	edit->selectAll() ;
}








