// MapParser.cpp: implementation of the CMapParser class.
//
//////////////////////////////////////////////////////////////////////

#include "mapParser.h"
#include <stdio.h>
#include <qstring.h>
#include <stdlib.h>

extern int errno ;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

int         nMaps ;
MapType    *pMaps ;

static int  nActMapCnt ;

CMapParser::CMapParser()
{

}

CMapParser::~CMapParser()
{

}

bool CMapParser::ParseMapFile(const char *sMapFile, QString &errMsg)
{
   FILE          *fp ;
   char           sSrcLine[200], sOrgText[50], sReplText[50], sApplyText[50], sErrorText[200] ;
   Type           type ;
	Side           side ;
	ScanRetType    ret ;
	int            nLine = 0 ;
	MapType        map ;

	errMsg = "" ;

	ResetMap() ;
//	gsMapFile = "" ;

   if ((fp = fopen(sMapFile, "r")) == NULL)
   {
      sprintf(sSrcLine, "Failed to read: '%s'\r\nError: %s", (const char *)sMapFile, strerror(errno)) ;
		errMsg = sSrcLine ;
      return false ;
   }

   while (fgets(sSrcLine, sizeof(sSrcLine), fp))
   {
		nLine++ ;

		ret = ScanLine(sSrcLine, sOrgText, sReplText, sApplyText, &type, &side, sErrorText) ;
      if (ret == ScanError)
      {
			ResetMap() ;
			fclose(fp) ;
			errMsg.sprintf("Error parsing map file:-\r\n\tSource:\t%s\r\n\tLine:\t%d\r\n\tError:\t%s", sMapFile, nLine, sErrorText) ;
			return false ;
      }
      if (ret == ScanNoData)
         continue ;

		if (!strlen(sOrgText))
		{
			ResetMap() ;
			fclose(fp) ;
			errMsg.sprintf("Error parsing map file:-\r\n\tSource:\t%s\r\n\tLine:\t%d\r\n\tError:\tInput text is 'nothing'.", sMapFile, nLine) ;
			return false ;
		}

		strcpy(map.eng, sOrgText) ;
		strcpy(map.bong_cons, sReplText) ;
		strcpy(map.bong_vow, sApplyText) ;
		map.type = type ;
		map.side = side ;
		AddMap(&map) ;
   }

   fclose(fp) ;

//	gsMapFile = sMapFile ;

   return true ;
}

void CMapParser::SkipSpace(char * sSrcText, int nSrcLen, int * index)
{
   while (*index < nSrcLen && (sSrcText[*index] == ' ' || sSrcText[*index] == '\t'))
      (*index)++ ;
}

ScanRetType CMapParser::ScanLine(char *sSrcLine, char *sOrgText, char *sReplText,
											char *sApplyText, Type *type, Side *side, char *sError)
{
   int      i, n ;

	*sOrgText = *sReplText = *sApplyText = '\0' ;

   n = strlen(sSrcLine) ;
   while (n > 0 && (sSrcLine[n-1] == '\r' || sSrcLine[n-1] == '\n'))
      sSrcLine[--n] = '\0' ;

   if (n == 0)
      return ScanNoData ;

   i = 0 ;
   SkipSpace(sSrcLine, n, &i) ;
   if (i == n)
      return ScanNoData ;

   while (ReadToken(sSrcLine, sOrgText, n, &i)) ;

/*
   SkipSpace(sSrcLine, n, &i) ;
   if (i == n)
	{
		sprintf(sError, "Unexpected end of line after original text!") ;
      return ScanError ;
	}
*/
   SkipSpace(sSrcLine, n, &i) ;
   if (i == n)
	{
		sprintf(sError, "Unexpected end of line while reading replacement text!") ;
      return ScanError ;
	}

   while (ReadToken(sSrcLine, sReplText, n, &i)) ;

   SkipSpace(sSrcLine, n, &i) ;
   if (i == n)
	{
		sprintf(sError, "Unexpected end of line while reading text to apply!") ;
      return ScanError ;
	}

   while (ReadToken(sSrcLine, sApplyText, n, &i)) ;

   SkipSpace(sSrcLine, n, &i) ;
   if (i == n)
	{
		sprintf(sError, "Unexpected end of line while reading map type!") ;
      return ScanError ;
	}

	if (!strnicmp(&sSrcLine[i], "vowel", 5))
	{
		*type = Vowel ;
		i += 5 ;
	}
	else if (!strnicmp(&sSrcLine[i], "consonant", 9))
	{
		*type = Cons ;
		i += 9 ;
	}
	else if (!strnicmp(&sSrcLine[i], "standalone", 10))
	{
		*type = StandAlone ;
		i += 10 ;
	}
	else if (!strnicmp(&sSrcLine[i], "stand_alone", 11))
	{
		*type = StandAlone ;
		i += 11 ;
	}
	else
	{
		sprintf(sError, "Syntax error near character no.  %d (%s)", i+1, &sSrcLine[i]) ;
      return ScanError ;
	}

   SkipSpace(sSrcLine, n, &i) ;
   if (i == n)
	{
		sprintf(sError, "Unexpected end of line while reading application side!") ;
      return ScanError ;
	}


	if (!strnicmp(&sSrcLine[i], "none", 4))
		*side = None ;
	else if (!strnicmp(&sSrcLine[i], "prefix", 6))
		*side = Pre ;
	else if (!strnicmp(&sSrcLine[i], "postfix", 7))
		*side = Post ;
	else if (!strnicmp(&sSrcLine[i], "both", 4))
		*side = Both ;
	else if (!strnicmp(&sSrcLine[i], "aroundpost", 10))
		*side = AroundPost ;
	else if (!strnicmp(&sSrcLine[i], "around", 6))
		*side = Around ;
	else
	{
		sprintf(sError, "Syntax error near character# %d (%s)", i+1, &sSrcLine[i]) ;
      return ScanError ;
	}

   return ScanDataFound ;
}

bool CMapParser::ReadToken(char * sSrcText, char * sDestText, int nSrcLen, int * index)
{
   char    buf[10] ;
   int     n, m = strlen(sDestText) ;

   if (*index >= nSrcLen || sSrcText[*index] == ' ' || sSrcText[*index] == '\t')
      return false ;

   switch (sSrcText[*index])
   {
      case '\\': 
         (*index)++ ;
         switch (sSrcText[*index])
         {
            case 'x': memcpy(buf, sSrcText+*index+1, 2) ; buf[2] = '\0' ;
                      sscanf(buf, "%x", &n) ;
                      *index += 3 ;
                      sDestText[m] = n ; sDestText[m+1] = '\0' ;
                      break ;
            case 'd': memcpy(buf, sSrcText+*index+1, 3) ; buf[3] = '\0' ;
                      sscanf(buf, "%d", &n) ;
                      *index += 4 ;
                      sDestText[m] = n ; sDestText[m+1] = '\0' ;
                      break ;
            case 'o': memcpy(buf, sSrcText+*index+1, 3) ; buf[3] = '\0' ;
                      sscanf(buf, "%o", &n) ;
                      *index += 4 ;
                      sDestText[m] = n ; sDestText[m+1] = '\0' ;
                      break ;
            default: sDestText[m] = sSrcText[*index] ; sDestText[m+1] = '\0' ; (*index)++ ; break ;
         }
         break ;
      case '-': (*index)++ ; break ;
		case '+': (*index)++ ; break ;
      default: sDestText[m] = sSrcText[*index] ; sDestText[m+1] = '\0' ; (*index)++ ; break ;
   }

	return true ;
}

void CMapParser::AddMap(MapType *map)
{
	if (nMaps == nActMapCnt)
	{
		if (pMaps == NULL)
			pMaps = (MapType *)malloc(MAP_GROW_SIZE * sizeof(MapType)) ;
		else
			pMaps = (MapType *)realloc(pMaps, (nActMapCnt+MAP_GROW_SIZE)*sizeof(MapType)) ;

		nActMapCnt += MAP_GROW_SIZE ;
	}

	pMaps[nMaps++] = *map ;
}

void CMapParser::ResetMap()
{
	if (nMaps != 0)
	{
		free(pMaps) ;
		nMaps = nActMapCnt = 0 ;
		pMaps = NULL ;
	}
}

