#include "soundTable.h"
#include <math.h>
#include "common.h"

CSoundTable::CSoundTable(QWidget *parent) : QFrame(parent, "IndSoundMap", 0, FALSE), m_defSize(300, 200)
{
	setFrameStyle(WinPanel | Sunken) ;
}

void CSoundTable::GetDimension(QSize &sz)
{
	int     h1, h2 ;
	int     row, col, r1 ;
	double  last = 999999999.99 ;

	if (nMaps == 0)
	{
		sz = m_defSize ;
		m_bIsValid = false ;
		return ;
	}

	QFontMetrics   f1(gEngFont) ;
	QFontMetrics   f2(gIndFont) ;

	m_bIsValid = true ;

	h1 = f1.height() ;
	h2 = f2.height() ;
	m_w1 = f1.maxWidth()*8 ;
	m_w2 = f2.maxWidth()*2 ;

	m_engBase = f1.descent()+1 ;
	m_indBase = f2.descent()+1 ;

	m_lineHeight = (h1 > h2) ? h1 : h2 ;

	m_lineHeight+= 2 ;
	m_itemWidth = m_w1 + m_w2*2 + HORZ_GAP + 1 ;

	m_row = m_col = 1 ;
	for (row = 1 ; row <= nMaps ; row++)
	{
		col = nMaps/row ;
		if (row*col != nMaps)
			col++ ;
		r1 = nMaps/col ;
		if (r1*col != nMaps)
			r1++ ;
		if (!IsMatrixBetter(r1, col, &last))
			break ;
		m_row = r1 ;
		m_col = col ;
	}

	sz.setWidth(m_col*m_itemWidth + 4) ;
	sz.setHeight(m_row*m_lineHeight+6) ;
}

bool CSoundTable::IsMatrixBetter(int nRow, int nCol, double *lastDif)
{
	double     d1 = (double)gScrnWidth/(double)gScrnHeight ;
	int        w = nCol*m_itemWidth, h = nRow*m_lineHeight ;
	double     d2 = (double)w/(double)h ;
	double     diff = d2 - d1 ;
	bool       ret = false ;

	if (fabs(diff) <= *lastDif)
		ret = true ;

	*lastDif = fabs(diff) ;

	return ret ;
}

void CSoundTable::paintEvent(QPaintEvent *paint)
{
	QFrame::paintEvent(paint) ;

	QPainter        p(this) ;
	int             i, j, k ;
	int             x, y, w = m_itemWidth, h=m_lineHeight ;
	QColor          c1(0xCF, 0xCF, 0xCF), c2(0xAF, 0xAF, 0xAD) ;
	QColor         *pclr ;
	QBrush          b1(c1), b2(c2) ;
	QBrush         *pbr ;

	if (!m_bIsValid)
	{
		p.drawText(0, 0, width(), height(), AlignHCenter | AlignVCenter, "No sounds to display") ;
		return ;
	}

	QRect         r ;

	for (i = 0, k = 0 ; i < m_col ; i++)
	{
		for (j = 0 ; j < m_row ; j++, k++)
		{
//			k = j*m_col + i ;

			x = 2 + i*m_itemWidth ;
			y = 2+j*m_lineHeight ;
			r.setX(x) ;
			r.setY(y) ;
			r.setWidth(w) ;
			r.setHeight(h) ;

			if (!r.intersects(paint->rect()))
				continue ;

			if (((i+j)/2)*2 == (i+j))
			{
				pclr = &c1 ;
				pbr = &b1 ;
			}
			else
			{
				pclr = &c2 ;
				pbr = &b2 ;
			}
			p.setBackgroundColor(*pclr) ;
			p.fillRect(x, y, m_itemWidth - 2, m_lineHeight-1, *pbr) ;

			if (k >= nMaps)
				continue ;

			p.setFont(gEngFont) ;
			p.drawText(x+HORZ_GAP/2, y+m_lineHeight-m_engBase, pMaps[k].eng) ;

			p.setFont(gIndFont) ;
			p.drawText(x+m_w1+HORZ_GAP/2, y+m_lineHeight-m_indBase, pMaps[k].bong_cons) ;
			p.drawText(x+m_w1+m_w2+HORZ_GAP/2, y+m_lineHeight-m_indBase, pMaps[k].bong_vow) ;
		}
	}

	for (i = 1 ; i < m_col ; i++)
	{
		x = 2+i*m_itemWidth - 1 ;
		p.drawLine(x, 0, x, height()) ;
	}
}





