#include "transliterator.h"
#include "mapParser.h"

CTransliterator::CTransliterator()
{
}

CTransliterator::~CTransliterator()
{
}

CTransliterator::CTransliterator(bool bForward)
{
	m_bForward = bForward ;
	m_bInPassThruMode = false ;
}


ConvRetType CTransliterator::ForwardConvert(const char * inBuf, char * outBuf)
{
	char          cons[50] ;
	char          append[50] ;
	char         *outptr ;
	const char   *inptr ;
	int           i, j, inputLen ;
	MapType       lastMap ;

	memset(outBuf, 0, strlen(inBuf)) ;

	inputLen = strlen(inBuf) ;

	memset(cons, 0, sizeof(cons)) ;
	outptr = outBuf ;
	inptr = inBuf ;
	memset(append, 0, sizeof(append)) ;

	while (*inptr)
	{
		for (i = 0 ; i < nMaps ; i++)
		{
			j = strlen(pMaps[i].eng) ;
			if (IsEqual(inptr, pMaps[i].eng))
				break ;
		}
		if (i == nMaps)  /* No matching bong sound */
		{
			if (strlen(cons))
			{
				strcat(outptr, cons) ;
				strcat(outptr, append) ;
				memset(cons, 0, sizeof(cons)) ;
				memset(append, 0, sizeof(append)) ;
			}

			StrAddCh(outptr, *inptr++) ;
			continue ;
		}

		inptr += j ;

		/*
		 * Let's say we found a consonant
		 */
		if (pMaps[i].type == Cons)
		{
			/*
			 * If there is a consonant in queue, handle it
			 * (not writing generic code now)
			 */
			if (strlen(cons))
			{
				int   nlen ;

				nlen = strlen(cons) ;
				if (lastMap.side == Pre)
				{
					cons[nlen - strlen(lastMap.bong_cons)] = '\0' ;
					strcat(cons, lastMap.bong_vow) ;
				}
				else if (lastMap.side == Both)
				{
					cons[nlen - strlen(lastMap.bong_cons)] = '\0' ;
					nlen = strlen(cons) ;
					cons[nlen] = lastMap.bong_vow[0] ;
					cons[nlen+1] = '\0' ;
				}
				else if (lastMap.side == Around)
				{
					cons[nlen - strlen(lastMap.bong_cons)] = '\0' ;
					strcat(append, lastMap.bong_vow) ;
				}
				else if (lastMap.side == AroundPost)
				{
					cons[nlen - strlen(lastMap.bong_cons)] = '\0' ;
					nlen = strlen(append) ;
					append[nlen] = lastMap.bong_vow[0] ;
					append[nlen+1] = '\0' ;
				}
			}

			/*
			 *  Now check if the consonant queue still has elements
			 */
			if (strlen(cons))
			{
				int   nlen ;

				if (pMaps[i].side == Post)
					strcat(cons, pMaps[i].bong_vow) ;
				else if (pMaps[i].side == Both || pMaps[i].side == AroundPost)
				{
					nlen = strlen(cons) ;
					cons[nlen] = pMaps[i].bong_vow[1] ;
					cons[nlen+1] = '\0' ;
				}
				else
					strcat(cons, pMaps[i].bong_cons) ;
			}
			else
			{
				/*
				 * Put the new consonant in queue
				 */
				strcpy(cons, pMaps[i].bong_cons) ;
			}
			lastMap = pMaps[i] ;
			continue ;
		}

		/*
		 * No, how about a stand-alone character
		 */
		if (pMaps[i].type == StandAlone)
		{
			/*
			 * If there is a consonant in queue, copy it to output
			 */
			if (strlen(cons))
			{
				strcat(outptr, cons) ;
				memset(cons, 0, sizeof(cons)) ;
			}
			strcat(outptr, pMaps[i].bong_cons) ;
			append[0] = '\0' ;

			continue ;
		}

		/*
		 * No, we found a vowel
		 */
		if (!strlen(cons))
		{
			/*
			 * No consonant in queue to apply vowel
			 * So, stand-alone vowel char
			 */
			strcat(outptr, pMaps[i].bong_cons) ;
			continue ;
		}
		/*
		 * There is a consonant in queue. Apply vowel to it.
		 */
		if (pMaps[i].side == Pre)
		{
			/*
			 * Vowel char comes first in bong
			 */
			strcat(outptr, pMaps[i].bong_vow) ;
			strcat(outptr, cons) ;
			strcat(outptr, append) ;
			append[0] = '\0' ;
			memset(cons, 0, sizeof(cons)) ;
			continue ;
		}
		if (pMaps[i].side == Post)
		{
			/*
			 * Vowel char comes last in bong
			 */
			strcat(outptr, cons) ;
			strcat(outptr, append) ;
			append[0] = '\0' ;
			strcat(outptr, pMaps[i].bong_vow) ;
			memset(cons, 0, sizeof(cons)) ;
			continue ;
		}
		if (pMaps[i].side == Both)
		{
			/*
			 * Vowel char comes on both sides of consonant in bong
			 */
			StrAddCh(outptr, pMaps[i].bong_vow[0]) ;
			strcat(outptr, cons) ;
			strcat(outptr, append) ;
			append[0] = '\0' ;
			StrAddCh(outptr, pMaps[i].bong_vow[1]) ;
			memset(cons, 0, sizeof(cons)) ;
			continue ;
		}
		/*
		 * SO, we found a vowel and a consonant is waiting in queue
		 * But the vowel is never applied on any side of the
		 * consonant. So, just copy them.
		 */
		strcat(outptr, cons) ;
		strcat(outptr, append) ;
		append[0] = '\0' ;
		strcat(outptr, pMaps[i].bong_cons) ;
		memset(cons, 0, sizeof(cons)) ;
	}
	strcat(outptr, cons) ;
	
	return NoSwitch ;
}

int CTransliterator::IsEqual(const char * eng, const char * map)
{
	const char * t1, *t2 ;

	for (t1 = eng, t2 = map ; *t1 && *t2 ; t1++, t2++)
		if (*t1 != *t2)
			return 0 ;

	if (*t2 && !*t1)
		return 0 ;

	return 1 ;
}

void CTransliterator::StrAddCh(char *str, char ch)
{
	char   buf[2] ;

	buf[0] = ch ;
	buf[1] = '\0' ;

	strcat(str, buf) ;
}

